#include "mainwindow.h"
#include "ui_mainwindow.h"

extern "C"
{
    #include "usblc8mpi.h"
}

void MainWindow::startpipethread(quint16 pxlcnt, qint8 bits16)
{
    this->ThPipe = new PipeThread(this);
    this->ThPipe->is16bit = bits16;
    this->ThPipe->BufSize = pxlcnt * (bits16 + 1);
    connect(this->ThPipe,SIGNAL(DataChanged(quint16*,qint32,qint32,quint32)),this,SLOT(OnDataChanged(quint16*,qint32,qint32,quint32)));
    connect(this->ThPipe, SIGNAL(finished()),this->ThPipe, SLOT(deleteLater()));
    this->ThPipe->start();
}

void MainWindow::stoppipethread()
{
    if (this->ThPipe->isRunning())
    {
        this->ThPipe->stopthread();
        this->ThPipe->wait();
    }
}

void MainWindow::syncparam()
{
    quint16 bcdversion;
    quint8 u8tmp;
    quint16 u16tmp;
    quint32 u32tmp;
    qint16  i16tmp;
    float ftmp;

    bcdversion = ls_getfwversion(ls_currentdeviceindex());

    if (bcdversion >= 0x0114) {
        ui->btn_trigtime->setEnabled(true);
        ui->ed_trigtime->setEnabled(true);
        if (ls_getsofttrigtime(u32tmp,1000) == 0) {
            ui->ed_trigtime->setText(QString::number(u32tmp));
        }
    } else {
        ui->ed_trigtime->setText("0");
        ui->btn_trigtime->setDisabled(true);
        ui->ed_trigtime->setDisabled(true);
    }

    if (ls_getstate(u8tmp,1000) == 0) {
        if (u8tmp == 0) ui->btn_start->setText("start");
        else ui->btn_start->setText("stop");
    }

    if (ls_getmode(u8tmp,1000) == 0) {
        ui->cb_mode->blockSignals(true);

        ui->cb_mode->clear();
        ui->cb_mode->addItem("ONE SHOT");
        ui->cb_mode->addItem("EXT. TRIGGER");
        ui->cb_mode->addItem("FREE RUNNING");
        ui->cb_mode->addItem("EXT. EXP. CTRL");

        if (bcdversion >= 0x0114) {
           ui->cb_mode->addItem("SOFT TRIGGER");
        }

        ui->cb_mode->setCurrentIndex(u8tmp);
        ui->cb_mode->blockSignals(false);
    }

    if (ls_getinttime(u32tmp,1000) == 0) {
        ui->ed_inttime->setText(QString::number(u32tmp));
    }

    if (ls_getextdelay(u32tmp,1000) == 0) {
        ui->ed_delay->setText(QString::number(u32tmp));
    }

    if (ls_getadcconfig(u16tmp,1000) == 0) {
        ui->cb_fullscale->blockSignals(true);
        ui->cb_fullscale->setCurrentIndex((u16tmp >> 7) & 0x01);
        ui->cb_fullscale->blockSignals(false);
    }

    if (ls_getadcpga1(u16tmp,1000) == 0) {
        ui->hs_gain->blockSignals(true);
        ui->hs_gain->setValue(u16tmp);
        ui->hs_gain->blockSignals(false);
        ui->lb_gain->setText(QString::number(6 / (1+(5*((63- (float)u16tmp)/63))),'f',4)+" V/V");
    }

    if (ls_getadcoffset1(u16tmp,1000) == 0) {
        i16tmp = u16tmp & 0xFF;
        if ((u16tmp & 0x0100) == 0x0100) i16tmp = i16tmp * (-1);

        ui->hs_offset->blockSignals(true);
        ui->hs_offset->setValue(i16tmp);
        ui->hs_offset->blockSignals(false);

        ftmp = (float)i16tmp * 1.2;
        if (ftmp < -300) ftmp = -300;
        if (ftmp > 300) ftmp = 300;
        ui->lb_offset->setText(QString::number(ftmp,'f',1)+" mV");
    }

    if (ls_getcfg1(u8tmp,1000) == 0) {
        ui->cb_imagenum->setCurrentIndex(u8tmp & 0x01);
        ui->cb_168BitMode->setCurrentIndex((u8tmp & 0x02) >> 1);
        this->is16bit = ui->cb_168BitMode->currentIndex();
        ui->sb_numframes->setValue(((u8tmp & 0x7C) >> 2)+1);
    }
}

void MainWindow::init_statusBar()
{
    QStatusBar * bar = ui-> statusBar;

    statusMsg = new QLabel;
    statusMsg-> setMinimumSize (350,20);
    statusMsg-> setFrameShape (QFrame :: WinPanel);
    statusMsg-> setFrameShadow (QFrame :: Sunken);

    statusFPS = new QLabel;
    statusFPS-> setMinimumSize (75,20);
    statusFPS-> setFrameShape (QFrame :: WinPanel);
    statusFPS-> setFrameShadow (QFrame :: Sunken);

    statusMEM = new QLabel;
    statusMEM-> setMinimumSize (80,20);
    statusMEM-> setFrameShape (QFrame :: WinPanel);
    statusMEM-> setFrameShadow (QFrame :: Sunken);

    bar-> addWidget (statusMsg);
    bar-> addWidget (statusFPS);
    bar-> addWidget (statusMEM);

    statusMsg->setText ("USB LINE CAMERA");
    statusFPS->setText ("FPS: 0");
    statusMEM->setText ("MEM: 0");
}

void MainWindow::init_chart(quint16 pixelcount, quint8 bits16)
{
    if (bits16 == 1){
        plot->setAxisScale( QwtPlot::yLeft, 0, 65535);
    } else {
        plot->setAxisScale( QwtPlot::yLeft, 0, 255);
    }
    plot->setAxisScale( QwtPlot::xBottom, 0, pixelcount-1);
    points->clear();
    for(int i=0;i<pixelcount;i++){
        points->push_back(QPointF(i,0));
    }
    seriesdata->setSamples(*points);
    plot->replot();
}

void MainWindow::create_chart(quint16 pixelcount)
{
    plot = new QwtPlot(ui->plotwidget);
    plot->setTitle( "USB Line Camera 8M" );
    plot->setCanvasBackground( Qt::white );
    plot->setAxisScale( QwtPlot::yLeft, 0, 65535);
    plot->setAxisScale( QwtPlot::xBottom, 0, pixelcount-1);

    grid = new QwtPlotGrid();
    grid->attach(plot);

    curve = new QwtPlotCurve("Curve 1");
    curve->setPen( Qt::blue, 0 ),
    curve->setRenderHint( QwtPlotItem::RenderAntialiased, true );

    seriesdata = new QwtPointSeriesData;
    points = new QVector<QPointF>;

    for(int i=0;i<pixelcount;i++){
        points->push_back(QPointF(i,0));
    }
    seriesdata->setSamples(*points);
    curve->setData(seriesdata);
    curve->attach(plot);

    plot->resize( 500, 320 );
    plot->replot();
}

void MainWindow::OnDataChanged(quint16* buf, qint32 BytesRead, qint32 BytesAvailable, quint32 fps)
{
    statusMEM->setText("MEM: "+QString::number(BytesAvailable));
    statusFPS->setText("FPS: "+QString::number(fps));

    points->clear();
    if (this->is16bit) {
        for(int i=0; i<BytesRead/2;i++){
            points->push_back(QPointF(i,buf[i]));
        }
    } else {
        for(int i=0; i<BytesRead/2;i=i+1){
            points->push_back(QPointF((i*2),(buf[i] & 0xff)));
            points->push_back(QPointF((i*2)+1,(buf[i] & 0xff00)>>8));
        }
    }
    seriesdata->setSamples(*points);

    //replot() is too slow..
    //10 ms refresh / replot rate
    if (PlotRefreshTimer.elapsed() > 10) {
        plot->replot();
        PlotRefreshTimer.restart();
    }
}

MainWindow::MainWindow(QWidget *parent) :
    QMainWindow(parent),
    ui(new Ui::MainWindow)
{
    ui->setupUi(this);
    init_statusBar();

    this->is16bit = 1;
    this->wpixelcount = 4096;
    this->packetlength = (this->wpixelcount * 2) * PACKETLENGTH_MULTIPLIER;
    ls_initialize(4*1024*1024,this->packetlength); //4MB PipeSize
    create_chart(this->wpixelcount);

    PlotRefreshTimer.start();
}

MainWindow::~MainWindow()
{
    close_device();
    delete ui;
}

void MainWindow::close_device()
{
    if (ls_currentdeviceindex()>= 0){
        this->stoppipethread();
        ls_closedevice();
        ui->btn_open->setText("open");
    }
}

void MainWindow::open_device(qint32 index)
{
    qint32 tmp_packetlength;
    quint16 tmp_sensortype;
    quint16 tmp_pixelcount;
    qint32 ierr;

    if (ls_currentdeviceindex()>= 0){
        close_device();
    } else {
        if (ls_devicecount() > 0){
            ierr = ls_opendevicebyindex(index);
            if (ierr == 0){
                ierr = ls_getpacketlength(tmp_packetlength,1000);
                if (ierr == 0) {
                    if (tmp_packetlength != (this->packetlength / PACKETLENGTH_MULTIPLIER)) {
                        this->packetlength = (tmp_packetlength * PACKETLENGTH_MULTIPLIER);
                        ls_closedevice();
                        ls_setpacketlength(this->packetlength);
                        ierr = ls_opendevicebyindex(index);
                        if (ierr != 0) {
                            statusMsg->setText("OPEN DEVICE: "+QString(ls_geterrorstring(ierr)));
                            return;
                        }
                    }
                } else statusMsg->setText("GET PACKETLENGTH: "+QString(ls_geterrorstring(ierr)));

                ierr = ls_getsensortype(tmp_sensortype,tmp_pixelcount,1000);
                if (ierr == 0) {
                    this->wsensortype = tmp_sensortype;
                    this->wpixelcount = tmp_pixelcount;
                } else statusMsg->setText("GET SENSORTYPE: "+QString(ls_geterrorstring(ierr)));
                this->syncparam();
                init_chart(this->wpixelcount,this->is16bit);
                this->startpipethread(this->wpixelcount,this->is16bit);
                ui->btn_open->setText("close");
            } else statusMsg->setText("OPEN DEVICE: "+QString(ls_geterrorstring(ierr)));
        }
    }
}

void MainWindow::on_btn_enum_clicked()
{
    quint16 cnt;

    this->close_device();
    ui->cb_devices->clear();
    cnt = ls_enumdevices();
    if (cnt > 0) {
        statusMsg->setText("ENUM DEVICES: "+QString::number(cnt)+" DEVICE(S) FOUND");
        for(int i=0;i<cnt;i++){
            ui->cb_devices->addItem(QString(ls_getproductname(i))+" "+QString(ls_getserialnumber(i)));
        }
        ui->cb_devices->setCurrentIndex(0);
        this->open_device(ui->cb_devices->currentIndex());
    } else statusMsg->setText("ENUM DEVICES: NO DEVICES FOUND");
}

void MainWindow::on_btn_open_clicked()
{
    this->open_device(ui->cb_devices->currentIndex());
}

void MainWindow::on_btn_start_clicked()
{
    quint8 state;
    quint8 mode;
    qint32 ierr;

    ierr = ls_getstate(state,1000);
    if (ierr == 0){
        state = (state & 0x01) ^ 0x01;
        ierr = ls_setstate(state,1000);
        if (ierr == 0){
            ls_getmode(mode,1000);
            if ((state == 0) || (mode == 0)) ui->btn_start->setText("start");
            else ui->btn_start->setText("stop");
        }
    }
    statusMsg->setText("SET STATE: "+QString(ls_geterrorstring(ierr)));
}

void MainWindow::on_btn_inttime_clicked()
{
    qint32 ierr;
    quint32 u32tmp;

    ierr = ls_setinttime(ui->ed_inttime->text().toInt(),1000);
    if (ierr == 0) {
        if (ls_getinttime(u32tmp,1000) == 0) {
            ui->ed_inttime->setText(QString::number(u32tmp));
        }
    }
    statusMsg->setText("SET INT.TIME: "+QString(ls_geterrorstring(ierr)));
}

void MainWindow::on_hs_gain_valueChanged(int value)
{
    qint32 ierr;
    float fgain;

    ierr = ls_setadcpga1(value,1000);
    statusMsg->setText("SET GAIN: "+QString(ls_geterrorstring(ierr)));

    fgain = 6 / (1+(5*((63- (float)value)/63)));
    ui->lb_gain->setText(QString::number(fgain,'f',4)+" V/V");
}

void MainWindow::on_hs_offset_valueChanged(int value)
{
    float newvalue;
    quint16 wvalue;
    quint16 sign;
    qint32 ierr;

    newvalue = (float)value * 1.2;
    if (newvalue < -300) newvalue = -300;
    if (newvalue > 300) newvalue = 300;

    if (value < 0) sign = 0x0100;
    else sign = 0;
    wvalue = abs(value) | sign;

    ierr = ls_setadcoffset1(wvalue,1000);
    statusMsg->setText("SET OFFSET: "+QString(ls_geterrorstring(ierr)));

    ui->lb_offset->setText(QString::number(newvalue,'f',1)+" mV");
}

void MainWindow::on_cb_mode_currentIndexChanged(int index)
{
    qint32 ierr;

    ierr = ls_setmode(index,1000);
    if ((ierr == 0) && (index == 0))
        ui->btn_start->setText("start");

    statusMsg->setText("SET MODE: "+QString(ls_geterrorstring(ierr)));
}

void MainWindow::on_cb_fullscale_currentIndexChanged(int index)
{
    qint32 ierr;

    ierr = ls_setadcconfig(index << 7,1000);
    statusMsg->setText("FULL SCALE: "+QString(ls_geterrorstring(ierr)));
}

void MainWindow::on_btn_savesettings_clicked()
{
    qint32 ierr;

    ierr = ls_savesettings(1000);
    statusMsg->setText("SAVE SETTINGS: "+QString(ls_geterrorstring(ierr)));
}

void MainWindow::on_btn_getinfo_clicked()
{
    qint32 ierr;
    quint16 u16tmp;
    qint32 i32tmp;
    quint32 u32tmp;
    quint16 sensor;
    quint16 pixel;
    int idx;

    idx = ui->cb_devices->currentIndex();

    ui->lb_vendor->setText(ls_getvendorname(idx));
    ui->lb_product->setText(ls_getproductname(idx));
    ui->lb_serial->setText(ls_getserialnumber(idx));

    u16tmp = ls_getfwversion(idx);
    ui->lb_mcu1version->setText(QString::number((u16tmp >> 8) & 0xFF)+"."+QString::number(u16tmp & 0xFF));

    ierr = ls_getsensortype(sensor,pixel,1000);
    if (ierr == 0) {
        ui->lb_sensor->setText("0x"+QString::number(sensor,16)+" <"+ls_getsensorname(sensor)+">");
        ui->lb_pixel->setText(QString::number(pixel));
    }

    ierr = ls_getmcusensortype(sensor,1000);
    if (ierr == 0) ui->lb_mcu2sensor->setText("0x"+QString::number(sensor,16)+" <"+ls_getsensorname(sensor)+">");

    ierr = ls_getpacketlength(i32tmp,1000);
    if (ierr == 0) ui->lb_mcu2packet->setText(QString::number(i32tmp)+" Bytes");
    else ui->lb_mcu2packet->setText("?0");

    ierr = ls_customfirmware(u32tmp,1000);
    if (ierr == 0) ui->lb_custom->setText(QString::number(u32tmp));
    else ui->lb_custom->setText("?0");

    u16tmp = ls_libversion();
    ui->lb_libversion->setText(QString::number((u16tmp >> 8) & 0xFF)+"."+QString::number(u16tmp & 0xFF));  //+QString::number(u16tmp & 0xFF,16));
}

void MainWindow::on_btn_delay_clicked()
{
    qint32 ierr;
    quint32 u32tmp;

    ierr = ls_setextdelay(ui->ed_delay->text().toInt(),0);
    if (ierr == 0) {
        if (ls_getextdelay(u32tmp,1000) == 0) {
            ui->ed_delay->setText(QString::number(u32tmp));
        }
    }
    statusMsg->setText("SET EXT.DELAY: "+QString(ls_geterrorstring(ierr)));
}

void MainWindow::on_pushButton_clicked()
{
    qint32 ierr;
    quint8 uccfg1;

    uccfg1 = (ui->sb_numframes->value() & 0x1F)-1;
    uccfg1 = (uccfg1 << 1) | (ui->cb_168BitMode->currentIndex() & 0x01);
    uccfg1 = (uccfg1 << 1) | (ui->cb_imagenum->currentIndex() & 0x01);
    ierr = ls_setcfg1(uccfg1,1000);
    if (ierr == 0)
    {
      //Changes takes effect after reboot - Power OFF/ON
    }
    statusMsg->setText("SET CFG1: "+QString(ls_geterrorstring(ierr)));
}

void MainWindow::on_btn_trigtime_clicked()
{
    qint32 ierr;
    quint32 u32tmp;

    ierr = ls_setsofttrigtime(ui->ed_trigtime->text().toInt(),0);
    if (ierr == 0) {
        if (ls_getsofttrigtime(u32tmp,1000) == 0) {
            ui->ed_trigtime->setText(QString::number(u32tmp));
        }
    }
    statusMsg->setText("SET TRIG TIME: "+QString(ls_geterrorstring(ierr)));
}
