#ifndef MAINWINDOW_H
#define MAINWINDOW_H

#include <QMainWindow>
#include <QLabel>
#include <qwt.h>
#include <qwt_plot_curve.h>
#include <qwt_plot.h>
#include <qwt_plot_grid.h>
#include <qwt_symbol.h>
#include <qwt_legend.h>
#include "pipethread.h"

#define PACKETLENGTH_MULTIPLIER 1


namespace Ui {
class MainWindow;
}

class MainWindow : public QMainWindow
{
    Q_OBJECT

public:
    explicit MainWindow(QWidget *parent = 0);
    ~MainWindow();
    PipeThread *ThPipe;

private slots:
    void on_btn_enum_clicked();

    void on_btn_open_clicked();

    void on_btn_start_clicked();

    void on_btn_inttime_clicked();

    void on_hs_gain_valueChanged(int value);

    void on_hs_offset_valueChanged(int value);

    void on_cb_mode_currentIndexChanged(int index);

    void on_cb_fullscale_currentIndexChanged(int index);

    void on_btn_savesettings_clicked();

    void on_btn_getinfo_clicked();

    void on_btn_delay_clicked();

    void on_pushButton_clicked();

    void on_btn_trigtime_clicked();

private:
    Ui::MainWindow *ui;
    QwtPlot *plot;
    QwtPlotGrid *grid;
    QwtPlotCurve *curve;
    QwtPointSeriesData* seriesdata;
    QVector<QPointF>* points;

    qint32 packetlength;
    quint16 wsensortype;
    quint16 wpixelcount;
    qint8 is16bit;

    QLabel * statusMsg;
    QLabel * statusFPS;
    QLabel * statusMEM;

    QTime PlotRefreshTimer;

    void init_statusBar ();
    void init_chart(quint16, quint8);
    void create_chart(quint16);

    void close_device();
    void open_device(qint32);
    void startpipethread(quint16, qint8);
    void stoppipethread();
    void syncparam();

public slots:
    void OnDataChanged(quint16*,qint32,qint32,quint32);
};

#endif // MAINWINDOW_H
